@@ -21,16 +21,28 @@ my class Date does Dateish {
     method !SET-SELF($!year,$!month,$!day,&!formatter,$!daycount = Int) { self }
 
     proto method new(|) {*}
-    multi method new(Date: Int() $year, Int() $month, Int() $day, :&formatter, *%_) {
-        (1..12).in-range($month,'Month');
-        (1 .. self.DAYS-IN-MONTH($year,$month)).in-range($day,'Day');
+    multi method new(Date: Int:D() $year, Int:D() $month, Int:D() $day, :&formatter, *%_) {
+        1 <= $month <= 12
+          || X::OutOfRange.new(:what<Month>,:got($month),:range<1..12>).throw;
+        1 <= $day <= self.DAYS-IN-MONTH($year,$month)
+          || X::OutOfRange.new(
+               :what<Day>,
+               :got($day),
+               :range("1..{self.DAYS-IN-MONTH($year,$month)}")
+             ).throw;
         self === Date
           ?? nqp::create(self)!SET-SELF($year,$month,$day,&formatter)
           !! self.bless(:$year,:$month,:$day,:&formatter,|%_)
     }
-    multi method new(Date: Int() :$year!, Int() :$month = 1, Int() :$day = 1, :&formatter, *%_) {
-        (1..12).in-range($month,'Month');
-        (1 .. self.DAYS-IN-MONTH($year,$month)).in-range($day,'Day');
+    multi method new(Date: Int:D() :$year!, Int:D() :$month = 1, Int:D() :$day = 1, :&formatter, *%_) {
+        1 <= $month <= 12
+          || X::OutOfRange.new(:what<Month>,:got($month),:range<1..12>).throw;
+        1 <= $day <= self.DAYS-IN-MONTH($year,$month)
+          || X::OutOfRange.new(
+               :what<Day>,
+               :got($day),
+               :range("1..{self.DAYS-IN-MONTH($year,$month)}")
+             ).throw;
         self === Date
           ?? nqp::create(self)!SET-SELF($year,$month,$day,&formatter)
           !! self.bless(:$year,:$month,:$day,:&formatter,|%_)
@@ -82,7 +94,7 @@ my class Date does Dateish {
             ),
             nqp::unbox_i(self.daycount)
           ),
-          ObjAt
+          ValueObjAt
         )
     }
 
@@ -157,7 +169,7 @@ my class Date does Dateish {
         $dt.day == $!day && $dt.month == $!month && $dt.year == $!year
     }
 
-    proto method DateTime()  { * }
+    proto method DateTime()  {*}
     multi method DateTime(Date:D:) { DateTime.new(:$!year, :$!month, :$!day) }
     multi method DateTime(Date:U:) { DateTime }
     method Date() { self }
