@@ -2,6 +2,7 @@
 # deep; a List may contain Scalar containers that can be assigned to. However,
 # it is not possible to shift/unshift/push/pop/splice/bind. A List is also
 # Positional, and so may be indexed.
+my class Array { ... }
 my class List does Iterable does Positional { # declared in BOOTSTRAP
     # class List is Cool
     #   The reified elements in the list so far (that is, those that we already
@@ -339,7 +340,7 @@ my class List does Iterable does Positional { # declared in BOOTSTRAP
         )
     }
 
-    method new(**@things) {
+    method new(**@things is raw) {
         my \list = nqp::create(self);
         my \iterbuffer = nqp::create(IterationBuffer);
         nqp::bindattr(list, List, '$!reified', iterbuffer);
@@ -378,22 +379,27 @@ my class List does Iterable does Positional { # declared in BOOTSTRAP
           Failure.new(X::Cannot::Lazy.new(:action('.sum'))),
           nqp::if(
             $!reified.DEFINITE && (my int $elems = self.elems),      # reifies
-            nqp::stmts(
-              (my $list := $!reified),
-              (my $sum = nqp::ifnull(nqp::atpos($list,0),0)),
-              (my int $i),
-              nqp::while(
-                nqp::islt_i($i = nqp::add_i($i,1),$elems),
-                ($sum = $sum + nqp::ifnull(nqp::atpos($list,$i),0))
+            nqp::if(
+              nqp::isgt_i($elems,2),
+              nqp::stmts(
+                (my $list := $!reified),
+                (my $sum = nqp::ifnull(nqp::atpos($list,0),0)),
+                (my int $i),
+                nqp::while(
+                  nqp::islt_i($i = nqp::add_i($i,1),$elems),
+                  ($sum = $sum + nqp::ifnull(nqp::atpos($list,$i),0))
+                ),
+                $sum
               ),
-              $sum
+              nqp::ifnull(nqp::atpos($!reified,0),0)
+                + nqp::ifnull(nqp::atpos($!reified,1),0)
             ),
             0
           )
         )
     }
 
-    proto method fmt(|) { * }
+    proto method fmt(|) {*}
     multi method fmt() {
         nqp::if(
           (my int $elems = self.elems),             # reifies
@@ -477,39 +483,41 @@ my class List does Iterable does Positional { # declared in BOOTSTRAP
         )
     }
 
-    multi method AT-POS(List:D: Int:D $pos) is raw {
-        nqp::if(
-          nqp::islt_i($pos,0),
-          Failure.new(X::OutOfRange.new(
-            :what($*INDEX // 'Index'), :got($pos), :range<0..^Inf>)),
-          nqp::if(
-            $!reified.DEFINITE,
-            nqp::ifnull(
-              nqp::atpos($!reified,$pos),
-              nqp::if(
-                ($!todo.DEFINITE && $!todo.reify-at-least(nqp::add_i($pos,1))),
-                nqp::ifnull(nqp::atpos($!reified,$pos),Nil),
-                Nil
-              )
-            ),
-            Nil
-          )
-        )
-    }
-
     multi method AT-POS(List:D: int $pos) is raw {
         nqp::if(
-          nqp::islt_i($pos,0),
+          nqp::isge_i($pos,0) && nqp::isconcrete($!reified),
+          nqp::ifnull(
+            nqp::atpos($!reified,$pos),
+            self!AT-POS-SLOW($pos)
+          ),
+          self!AT-POS-SLOW($pos)
+        )
+    }
+    # because this is a very hot path, we copied the code from the int candidate
+    multi method AT-POS(List:D: Int:D $pos) is raw {
+        nqp::if(
+          nqp::isge_i($pos,0) && nqp::isconcrete($!reified),
+          nqp::ifnull(
+            nqp::atpos($!reified,$pos),
+            self!AT-POS-SLOW($pos)
+          ),
+          self!AT-POS-SLOW($pos)
+        )
+    }
+
+    method !AT-POS-SLOW(\pos) is raw {
+        nqp::if(
+          nqp::islt_i(pos,0),
           Failure.new(X::OutOfRange.new(
-            :what($*INDEX // 'Index'), :got($pos), :range<0..^Inf>)),
+            :what($*INDEX // 'Index'), :got(pos), :range<0..^Inf>)),
           nqp::if(
-            $!reified.DEFINITE,
-            nqp::if(
-              nqp::islt_i($pos,nqp::elems($!reified)),
-              nqp::atpos($!reified,$pos),
+            nqp::isconcrete($!reified),
+            nqp::ifnull(
+              nqp::atpos($!reified,pos),
               nqp::if(
-                ($!todo.DEFINITE && $!todo.reify-at-least(nqp::add_i($pos,1))),
-                nqp::ifnull(nqp::atpos($!reified,$pos),Nil),
+                nqp::isconcrete($!todo)
+                  && $!todo.reify-at-least(nqp::add_i(pos,1)),
+                nqp::ifnull(nqp::atpos($!reified,pos),Nil),
                 Nil
               )
             ),
@@ -518,10 +526,14 @@ my class List does Iterable does Positional { # declared in BOOTSTRAP
         )
     }
 
-    method BIND-POS(List:D: Int:D \pos, \what) is raw {
+    method ASSIGN-POS(List:D: Int:D \pos, \what) is raw {
         nqp::iscont(self.AT-POS(pos))
-          ?? nqp::bindpos($!reified,nqp::unbox_i(pos),what)
-          !! X::Bind.new.throw
+          ?? (nqp::atpos($!reified,nqp::unbox_i(pos)) = what)
+          !! X::Assignment::RO.new(value => self).throw
+    }
+
+    method BIND-POS(List:D: Int:D \pos, \what) {
+        X::Bind.new.throw
     }
 
     multi method EXISTS-POS(List:D: int $pos) {
@@ -694,6 +706,7 @@ my class List does Iterable does Positional { # declared in BOOTSTRAP
 
     multi method list(List:D:) { self }
 
+    # We don't sink contents by design https://github.com/rakudo/rakudo/issues/1393
     method sink(--> Nil) { }
 
     multi method values(List:D:) {
@@ -724,7 +737,7 @@ my class List does Iterable does Positional { # declared in BOOTSTRAP
 
     # Store in List targets containers with in the list. This handles list
     # assignments, like ($a, $b) = foo().
-    proto method STORE(|) { * }
+    proto method STORE(|) {*}
     multi method STORE(List:D: Iterable:D \iterable) {
         # First pass -- scan lhs containers and pick out scalar versus list
         # assignment. This also reifies the RHS values we need, and deconts
@@ -796,14 +809,16 @@ my class List does Iterable does Positional { # declared in BOOTSTRAP
     }
 
     multi method gist(List:D:) {
-        self.gistseen('List', {
-            '(' ~ self.map( -> $elem {
+        self.gistseen(self.^name, {
+            (nqp::istype(self,Array) ?? '[' !! '(')
+            ~ self.map( -> $elem {
                 given ++$ {
                     when 101 { '...' }
                     when 102 { last }
                     default  { $elem.gist }
                 }
-            }).join(' ') ~ ')'
+            }).join(' ')
+            ~ (nqp::istype(self,Array) ?? ']' !! ')')
         })
     }
 
@@ -932,7 +947,7 @@ my class List does Iterable does Positional { # declared in BOOTSTRAP
         )
     }
 
-    proto method pick(|) is nodal { * }
+    proto method pick(|) is nodal {*}
     multi method pick(List:D:) {
         self.is-lazy
          ?? Failure.new(X::Cannot::Lazy.new(:action('.pick from')))
@@ -993,15 +1008,15 @@ my class List does Iterable does Positional { # declared in BOOTSTRAP
         }.new(self,$elems,$number))
     }
 
-    proto method roll(|) is nodal { * }
-    multi method roll() {
+    proto method roll(|) is nodal {*}
+    multi method roll(List:D:) {
         self.is-lazy
           ?? Failure.new(X::Cannot::Lazy.new(:action('.roll from')))
           !! (my Int $elems = self.elems)
             ?? nqp::atpos($!reified, $elems.rand.floor)
             !! Nil
     }
-    multi method roll(Whatever) {
+    multi method roll(List:D: Whatever) {
         nqp::if(
           self.is-lazy,
           X::Cannot::Lazy.new(:action('.roll from')).throw,
@@ -1014,7 +1029,7 @@ my class List does Iterable does Positional { # declared in BOOTSTRAP
           ))
         )
     }
-    multi method roll(\number) {
+    multi method roll(List:D: \number) {
         number == Inf
           ?? self.roll(*)
           !! self.is-lazy
@@ -1042,6 +1057,15 @@ my class List does Iterable does Positional { # declared in BOOTSTRAP
                              IterationEnd
                          }
                      }
+                     method push-all($target --> IterationEnd) {
+                         nqp::while(
+                             $!todo,
+                             nqp::stmts(
+                                 ($target.push(nqp::atpos($!list,$!elems.rand.floor))),
+                                 ($!todo = $!todo - 1)
+                             )
+                         )
+                     }
                  }.new(self,number.Int))
               !! Seq.new(Rakudo::Iterator.Empty)
     }
@@ -1120,7 +1144,21 @@ my class List does Iterable does Positional { # declared in BOOTSTRAP
     multi method combinations(Range:D $ofrange) {
         nqp::stmts(
           (my int $elems = self.elems),      # reifies
-          $ofrange.int-bounds(my int $i, my int $to),
+          nqp::if(
+            $ofrange.is-int,
+            $ofrange.int-bounds(my int $i, my int $to),
+            nqp::stmts(
+              nqp::unless(
+                $ofrange.min < 0,   # $i already 0 if not
+                ($i = $ofrange.min + $ofrange.excludes-min)
+              ),
+              nqp::if(
+                $ofrange.max > $elems,
+                ($to = $elems),
+                ($to = $ofrange.max - $ofrange.excludes-max)
+              )
+            )
+          ),
           ($i = nqp::if(nqp::islt_i($i,0),-1,nqp::sub_i($i,1))),
           nqp::if(nqp::isgt_i($to,$elems),($to = $elems)),
           Seq.new(
@@ -1142,11 +1180,9 @@ my class List does Iterable does Positional { # declared in BOOTSTRAP
 
     proto method permutations(|) is nodal {*}
     multi method permutations() {
-        Seq.new(
-          Rakudo::Iterator.ListIndexes(
-            self, Rakudo::Iterator.Permutations( self.elems, 1)
-          )
-        )
+        my \perm-iter = Rakudo::Iterator.Permutations: self.elems, 1;
+        Seq.new: Rakudo::Iterator.delegate-iterator-opt-methods:
+            perm-iter, Rakudo::Iterator.ListIndexes: self, perm-iter
     }
 
     method join(List:D: Str(Cool) $separator = '') is nodal {
@@ -1382,9 +1418,11 @@ my class List does Iterable does Positional { # declared in BOOTSTRAP
               nqp::stmts(
                 (my $iterator := Rakudo::Iterator.ReifiedList(self)),
                 nqp::if(
-                  nqp::istype($n,Callable)
-                    && nqp::isgt_i((my $skip := -($n(0).Int)),0),
-                  $iterator.skip-at-least($skip),
+                  nqp::istype($n,Callable),
+                  nqp::if(
+                    nqp::isgt_i((my $skip := -($n(0).Int)),0),
+                    $iterator.skip-at-least($skip)
+                  ),
                   nqp::unless(
                     nqp::istype($n,Whatever) || $n == Inf,
                     $iterator.skip-at-least(nqp::elems($!reified) - $n)
@@ -1477,24 +1515,26 @@ multi flat(Iterable \a)    {     a.flat }
 
 sub cache(+@l) { @l }
 
-proto sub infix:<xx>(|) { * }
+proto sub infix:<xx>(|) {*}
 multi sub infix:<xx>() { Failure.new("No zero-arg meaning for infix:<xx>") }
 multi sub infix:<xx>(Mu \x) { x }
-multi sub infix:<xx>(&x, Num() $n) {
+multi sub infix:<xx>(&x, Num:D() $n) {
     infix:<xx>(&x, $n == Inf ?? Whatever !! $n.Int);
 }
 multi sub infix:<xx>(&x, Whatever) {
     Seq.new(Rakudo::Iterator.Callable-xx-Whatever(&x))
 }
-multi sub infix:<xx>(&x, Int $n) {
-    my int $todo = $n + 1;
-    my Mu $pulled;
+multi sub infix:<xx>(&x, Bool:D $b) {
+    $b ?? infix:<xx>(&x, 1) !! EmptySeq
+}
+multi sub infix:<xx>(&x, Int:D $n) {
+    my int $todo = $n;
     my Mu $list := nqp::create(IterationBuffer);
     nqp::while(
-      nqp::isgt_i($todo = nqp::sub_i($todo,1),0),
+      nqp::isgt_i($todo = nqp::sub_i($todo,1),-1),
       nqp::if(
-        nqp::istype(($pulled := &x.()),Slip),
-        (nqp::push($list,$_) for $pulled),
+        nqp::istype((my $pulled := x()),Slip),
+        $pulled.iterator.push-all($list),
         nqp::if(
           nqp::istype($pulled,Seq),
           nqp::push($list,$pulled.cache),
@@ -1504,7 +1544,7 @@ multi sub infix:<xx>(&x, Int $n) {
     );
     Seq.new(Rakudo::Iterator.ReifiedList($list))
 }
-multi sub infix:<xx>(Mu \x, Num() $n) {
+multi sub infix:<xx>(Mu \x, Num:D() $n) {
     Seq.new(nqp::if(
       $n == Inf,
       Rakudo::Iterator.UnendingValue(x),
@@ -1514,11 +1554,14 @@ multi sub infix:<xx>(Mu \x, Num() $n) {
 multi sub infix:<xx>(Mu \x, Whatever) {
     Seq.new(Rakudo::Iterator.UnendingValue(x))
 }
+multi sub infix:<xx>(Mu \x, Bool:D $b) {
+    $b ?? Seq.new(Rakudo::Iterator.OneValue(x)) !! EmptySeq
+}
 multi sub infix:<xx>(Mu \x, Int:D $n) is pure {
     Seq.new(Rakudo::Iterator.OneValueTimes(x,$n))
 }
 
-proto sub reverse(|)   { * }
+proto sub reverse(|)   {*}
 multi sub reverse(@a)  { @a.reverse }
 multi sub reverse(+@a) { @a.reverse }
 
