@@ -3,14 +3,16 @@ my role  IO { ... }
 my class IO::Handle { ... }
 my class IO::Path { ... }
 my class Rakudo::Metaops { ... }
+
+my class X::Assignment::ToShaped { ... }
 my class X::Cannot::Lazy { ... }
-my class X::IllegalOnFixedDimensionArray { ... };
-my class X::Assignment::ToShaped { ... };
-my class X::Str::Sprintf::Directives::BadType { ... };
-my class X::Str::Sprintf::Directives::Count { ... };
-my class X::Str::Sprintf::Directives::Unsupported { ... };
+my class X::IllegalDimensionInShape { ... }
+my class X::IllegalOnFixedDimensionArray { ... }
+my class X::Localizer::NoContainer { ... }
+my class X::Str::Sprintf::Directives::BadType { ... }
+my class X::Str::Sprintf::Directives::Count { ... }
+my class X::Str::Sprintf::Directives::Unsupported { ... }
 my class X::TypeCheck { ... }
-my class X::IllegalDimensionInShape { ... };
 
 my class Rakudo::Internals {
 
@@ -165,6 +167,51 @@ my class Rakudo::Internals {
         )
     }
 
+    # Helper method for prefix:<let>/prefix:<temp>, which really do the same
+    # thing apart from where they store data.  Takes the IterationBuffer in
+    # which to save data, the container to be inspected, and the type of op
+    # for any error messaging.
+    method TEMP-LET(\restore, \cont, str $localizer) is raw {
+        nqp::stmts(
+          (my int $i = nqp::elems(restore)),
+          nqp::while(
+            nqp::isgt_i($i,0),
+            nqp::if(
+              nqp::eqaddr(nqp::atpos(restore,($i = nqp::sub_i($i,2))),cont),
+              (return-rw cont)
+            )
+          ),
+          nqp::if(
+            nqp::istype(cont,Failure),
+            cont.exception.throw,
+            nqp::stmts(
+              nqp::push(restore,cont),
+              nqp::if(
+                nqp::iscont(cont),
+                nqp::push(restore,nqp::decont(cont)),
+                nqp::if(
+                  nqp::istype(cont,Array),
+                  nqp::push(restore,cont.clone),
+                  nqp::if(
+                    nqp::istype(cont,Hash),
+                    nqp::push(restore,
+                      nqp::p6bindattrinvres(
+                        Hash.^parameterize(Mu,Mu).new,
+                        Hash, '$!descriptor',
+                        nqp::getattr(cont, Hash, '$!descriptor')).STORE: cont),
+                    nqp::stmts(
+                      nqp::pop(restore),  # lose the erroneously pushed value
+                      X::Localizer::NoContainer.new(:$localizer).throw
+                    )
+                  )
+                )
+              )
+            )
+          ),
+          cont
+        )
+    }
+
     # fast whitespace trim: str to trim, str to store trimmed str
     method TRIM(\string, \trimmed --> Nil) {
         my int $pos  = nqp::chars(string) - 1;
@@ -225,7 +272,11 @@ my class Rakudo::Internals {
       'ascii',           'ascii',
       'iso-8859-1',      'iso-8859-1',
       'windows-1252',    'windows-1252',
-      # with dash
+      'windows-1251',    'windows-1251',
+      # windows without dash
+      'windows1251',    'windows-1252',
+      'windows1252',    'windows-1251',
+      # utf with dash
       'utf-8',           'utf8',
       'utf-16',          'utf16',
       'utf-32',          'utf32',
@@ -552,7 +603,14 @@ my class Rakudo::Internals {
     method initialize-sprintf-handler(--> Nil) {
         class SprintfHandler {
             method mine($x) { nqp::reprname($x) eq "P6opaque"; }
-            method int($x) { $x.Int }
+
+            proto method int(|) {*}
+            multi method int(Mu:D \n) { n.Int }
+            multi method int(Mu:U \n) { n.Numeric.Int }
+
+            proto method float(|) {*}
+            multi method float(Numeric:D \n) { n }
+            multi method float(Mu \n) { n.Numeric }
         }
         unless $sprintfHandlerInitialized {
             nqp::sprintfaddargumenthandler(SprintfHandler.new);
@@ -560,6 +618,12 @@ my class Rakudo::Internals {
         }
     }
 
+    method SHORT-GIST(\thing) {
+        nqp::if(
+          nqp::isgt_i(nqp::chars(my str $gist = thing.gist), 23),
+          nqp::concat(nqp::substr($gist, 0, 20), '...'),
+          $gist);
+    }
     method SUBSTR-START-OOR(\from,\max) {
         X::OutOfRange.new(
           :what('Start argument to substr'),
@@ -635,7 +699,7 @@ my class Rakudo::Internals {
     }
 
     my num $init-time-num = nqp::time_n;
-    method INITTIME() { $init-time-num }
+    method INITTIME() is raw { $init-time-num }
 
     my $init-thread := nqp::currentthread();
     method INITTHREAD() { $init-thread }
@@ -1265,24 +1329,23 @@ my class Rakudo::Internals {
         my \payload := nqp::getpayload($vmex);
         if nqp::elems(payload) == 1 {
             if nqp::existskey(payload, 'BAD_TYPE_FOR_DIRECTIVE') {
-                X::Str::Sprintf::Directives::BadType.new(
+                X::Str::Sprintf::Directives::BadType.new:
                     type      => nqp::atkey(nqp::atkey(payload, 'BAD_TYPE_FOR_DIRECTIVE'), 'TYPE'),
                     directive => nqp::atkey(nqp::atkey(payload, 'BAD_TYPE_FOR_DIRECTIVE'), 'DIRECTIVE'),
-                ).throw
             }
-            if nqp::existskey(payload, 'BAD_DIRECTIVE') {
-                X::Str::Sprintf::Directives::Unsupported.new(
+            elsif nqp::existskey(payload, 'BAD_DIRECTIVE') {
+                X::Str::Sprintf::Directives::Unsupported.new:
                     directive => nqp::atkey(nqp::atkey(payload, 'BAD_DIRECTIVE'), 'DIRECTIVE'),
                     sequence  => nqp::atkey(nqp::atkey(payload, 'BAD_DIRECTIVE'), 'SEQUENCE'),
-                ).throw
             }
-            if nqp::existskey(payload, 'DIRECTIVES_COUNT') {
-                X::Str::Sprintf::Directives::Count.new(
+            elsif nqp::existskey(payload, 'DIRECTIVES_COUNT') {
+                X::Str::Sprintf::Directives::Count.new:
                     args-have => nqp::atkey(nqp::atkey(payload, 'DIRECTIVES_COUNT'), 'ARGS_HAVE'),
                     args-used => nqp::atkey(nqp::atkey(payload, 'DIRECTIVES_COUNT'), 'ARGS_USED'),
-                ).throw
             }
+            else { exception }
         }
+        else { exception }
     }
 
 #- start of generated part of succ/pred ---------------------------------------
@@ -1436,7 +1499,7 @@ my class Rakudo::Internals {
         $target
     }
 
-    proto method coremap(|) { * }
+    proto method coremap(|) {*}
 
     multi method coremap(\op, Associative \h, Bool :$deep) {
         my @keys = h.keys;
@@ -1535,13 +1598,15 @@ my constant $?BITS = nqp::isgt_i(nqp::add_i(2147483648, 1), 0) ?? 64 !! 32;
         unless $the-end-is-done {
             $the-end-locker.protect: {
                 unless $the-end-is-done {
-                    my $comp := nqp::getcomp('perl6');
-                    my $end  := nqp::getcurhllsym('@END_PHASERS');
+                    my $end := nqp::getcurhllsym('@END_PHASERS');
+                    my @exceptions;
                     while nqp::elems($end) {           # run all END blocks
-                        my $result := nqp::shift($end)();
-                        $result.sink if nqp::can($result,'sink');
-                        CATCH { $comp.handle-exception($_) }
-                        CONTROL { $comp.handle-control($_) }
+                        quietly {
+                            my $result := nqp::shift($end)();
+                            nqp::isfalse(nqp::isnull($result))
+                                && nqp::can($result, 'sink') && $result.sink;
+                            CATCH { default { @exceptions.push($_) } }
+                        }
                     }
 #?if moar
                     # close all open files
@@ -1549,6 +1614,10 @@ my constant $?BITS = nqp::isgt_i(nqp::add_i(2147483648, 1), 0) ?? 64 !! 32;
                       'close-all-open-handles'
                     )(IO::Handle);
 #?endif
+                    if @exceptions {
+                        note "Some exceptions were thrown in END blocks:";
+                        note "  $_.^name(): $_.message()" for @exceptions;
+                    }
                     nqp::not_i(($the-end-is-done = 1)); # we're really done now
                 }
             }
