@@ -37,7 +37,7 @@ my role Real does Numeric {
     method floor() { self.Bridge.floor }
     method ceiling() { self.Bridge.ceiling }
 
-    proto method round(|) { * }
+    proto method round(|) {*}
     multi method round(Real:D:) {
         (self + 1/2).floor; # Rat NYI here, so no .5
     }
@@ -107,11 +107,11 @@ my role Real does Numeric {
                 for @frac_digits-1 ... 0 -> $x {
                     last if ++@frac_digits[$x] < $base;
                     @frac_digits[$x] = 0;
-                    $int_part++ if $x == 0
+                    ++$int_part if $x == 0
                 }
             }
             else {
-                $int_part++;
+                ++$int_part;
             }
         }
         my Str $r = $int_part.base($base);
@@ -120,7 +120,11 @@ my role Real does Numeric {
         $int_part == 0 && self < 0 ?? '-' ~ $r !! $r;
     }
 
-    method Real(Real:D:) { self }
+    multi method Real(Real:D:) { self }
+    multi method Real(Real:U:) {
+        self.Mu::Real; # issue a warning;
+        self.new
+    }
     method Bridge(Real:D:) { self.Num }
     method Int(Real:D:) { self.Bridge.Int }
     method Num(Real:D:) { self.Bridge.Num }
@@ -174,7 +178,7 @@ multi sub truncate(Real:D $x) { $x.truncate }
 multi sub truncate(Cool:D $x) { $x.Numeric.truncate }
 
 
-proto sub atan2($, $?)    { * }
+proto sub atan2($, $?)    {*}
 multi sub atan2(Real \a, Real \b = 1e0) { a.Bridge.atan2(b.Bridge) }
 # should really be (Cool, Cool), and then (Cool, Real) and (Real, Cool)
 # candidates, but since Int both conforms to Cool and Real, we'd get lots
