@@ -15,7 +15,7 @@ class VM does Systemic {
 
     submethod BUILD(
       :$!config,
-      :$!desc = Str,
+      :$desc,
 #?if jvm
       :$!properties,
 #?endif
@@ -23,6 +23,7 @@ class VM does Systemic {
     ) {
 #?if moar
         $!name           = 'moar';
+        $!desc           = $desc // 'Short for "Metamodel On A Runtime", MoarVM is a modern virtual machine built for the Rakudo Perl 6 compiler and the NQP Compiler Toolchain.';
         $!auth           = "The MoarVM Team";
         $!version        = Version.new($!config<version> // "unknown");
         $!prefix         = $!config<prefix>;
@@ -31,6 +32,7 @@ class VM does Systemic {
 #?endif
 #?if jvm
         $!name           = 'jvm';
+        $!desc           = $desc // 'The Java Virtual Machine';
         $!auth           = $!properties<java.vendor> // "unknown";
         $!version        = Version.new($!properties<java.specification.version> // "unknown");
         $!prefix         = $!properties<perl6.prefix>;
@@ -69,7 +71,7 @@ class VM does Systemic {
           !! $platform-name.IO
     }
 
-    proto method osname(|) { * }
+    proto method osname(|) {*}
     multi method osname(VM:U:) {
 #?if jvm
         nqp::lc(nqp::atkey(nqp::jvmgetproperties,'os.name'))
@@ -89,9 +91,8 @@ class VM does Systemic {
 }
 
 sub INITIALIZE-A-VM-NOW() {
-    my $desc := DateTime.now.Str;
 #?if moar
-    VM.new(:config(nqp::backendconfig),:$desc);
+    VM.new(:config(nqp::backendconfig));
 #?endif
 #?if jvm
     my $config := do {
@@ -116,7 +117,7 @@ sub INITIALIZE-A-VM-NOW() {
         }
         %PROPS;
     }
-    VM.new(:$config,:$desc,:$properties);
+    VM.new(:$config,:$properties);
 #?endif
 }
 
