@@ -1,20 +1,17 @@
 my class IO::ArgFiles { ... }
 
-proto sub print(|) { * }
-multi sub print(Str:D \x) {
-    $*OUT.print(x);
-}
-multi sub print(\x) {
-    $*OUT.print(x.Str);
-}
-multi sub print(**@args is raw) {
-    my str $str;
-    $str = nqp::concat($str,nqp::unbox_s(.Str)) for @args;
-    $*OUT.print($str);
-}
+proto sub print(|) {*}
+multi sub print(--> True) { }    # nothing to do
+multi sub print(Junction:D \j)  { j.THREAD(&print) }
+multi sub print(Str:D \x)       { $*OUT.print(x) }
+multi sub print(\x)             { $*OUT.print(x.Str) }
+multi sub print(**@args is raw) { $*OUT.print: @args.join }
 
-proto sub say(|) { * }
+proto sub say(|) {*}
 multi sub say() { $*OUT.print-nl }
+multi sub say(Junction:D \j) {
+    j.THREAD(&say)
+}
 multi sub say(Str:D \x) {
     my $out := $*OUT;
     $out.print(nqp::concat(nqp::unbox_s(x),$out.nl-out));
@@ -33,8 +30,11 @@ multi sub say(**@args is raw) {
     $out.print(nqp::concat($str,$out.nl-out));
 }
 
-proto sub put(|) { * }
+proto sub put(|) {*}
 multi sub put() { $*OUT.print-nl }
+multi sub put(Junction:D \j) {
+    j.THREAD(&put)
+}
 multi sub put(Str:D \x) {
     my $out := $*OUT;
     $out.print(nqp::concat(nqp::unbox_s(x),$out.nl-out));
@@ -44,20 +44,18 @@ multi sub put(\x) {
     $out.print(nqp::concat(nqp::unbox_s(x.Str),$out.nl-out));
 }
 multi sub put(**@args is raw) {
-    my str $str;
-    my $iter := @args.iterator;
-    nqp::until(
-      nqp::eqaddr(($_ := $iter.pull-one), IterationEnd),
-      $str = nqp::concat($str, nqp::unbox_s(.Str)));
     my $out := $*OUT;
-    $out.print(nqp::concat($str,$out.nl-out));
+    $out.print: @args.join ~ $out.nl-out
 }
 
-proto sub note(|) { * }
+proto sub note(|) {*}
 multi sub note() {
     my $err := $*ERR;
     $err.print(nqp::concat("Noted",$err.nl-out));
 }
+multi sub note(Junction:D \j) {
+    j.THREAD(&note)
+}
 multi sub note(Str:D \x) {
     my $err := $*ERR;
     $err.print(nqp::concat(nqp::unbox_s(x),$err.nl-out));
@@ -86,34 +84,35 @@ multi sub prompt($msg) {
     $*IN.get;
 }
 
-proto sub dir(|) { * }
+proto sub dir(|) {*}
 multi sub dir(*%_) { $*SPEC.curdir.IO.dir(:!absolute, |%_) }
 multi sub dir(IO::Path:D $path, |c) { $path.dir(|c) }
 multi sub dir(IO()       $path, |c) { $path.dir(|c) }
 
-proto sub open(|) { * }
+proto sub open(|) {*}
 multi sub open(IO() $path, |c) { IO::Handle.new(:$path).open(|c) }
 
-proto sub lines(|) { * }
+proto sub lines(|) {*}
 multi sub lines($what = $*ARGFILES, |c) { $what.lines(|c) }
 
-proto sub words(|) { * }
+proto sub words(|) {*}
 multi sub words($what = $*ARGFILES, |c) { $what.words(|c) }
 
-proto sub get  (|) { * }
+proto sub get  (|) {*}
 multi sub get  (IO::Handle:D $fh = $*ARGFILES) { $fh.get  }
 
-proto sub getc (|) { * }
+proto sub getc (|) {*}
 multi sub getc (IO::Handle:D $fh = $*ARGFILES) { $fh.getc }
 
-proto sub close(|) { * }
+proto sub close(|) {*}
 multi sub close(IO::Handle:D $fh) { $fh.close }
+multi sub close(Channel:D $channel) { $channel.close }
 
-proto sub slurp(|) { * }
+proto sub slurp(|) {*}
 multi sub slurp(IO::Handle:D $fh = $*ARGFILES, |c) { $fh.slurp(|c) }
 multi sub slurp(IO() $path, |c) { $path.slurp(|c) }
 
-proto sub spurt(|) { * }
+proto sub spurt(|) {*}
 multi sub spurt(IO::Handle:D $fh,   |c) { $fh  .spurt(|c) }
 multi sub spurt(IO()       $path, |c) { $path.spurt(|c) }
 
